import { MatchService } from '../../service/platform/match.service';
import { CVO_match, FVO_match, QVO_match, UVAO_match } from '../../interface/platform/match.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { Controller, Inject, Get, Param, Post, Body, Del, Query } from '@midwayjs/core';
import { cloneDeep, get, omit, pick } from 'lodash';
import { ServiceError, ErrorCode } from '../../error/service.error';
import { BaseController } from '../../frame/BaseController';
import { ServiceUtilService } from '../../service/serviceUtil.service';
import { MatchExtService } from '../../service/match/matchExt.service';
const namePrefix = '创新大赛';
@ApiTags(['创新大赛'])
@Controller('/match', { tagName: namePrefix })
export class MatchController implements BaseController {
  @Inject()
  service: MatchService;
  @Inject()
  serviceUtil: ServiceUtilService;
  @Inject()
  matchExtService: MatchExtService;
  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_match })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others: any = pick(query, ['skip', 'limit']);
    others.order = { order_num: 'ASC' };
    const { data: list, total } = await this.service.query(qobj, others);
    const fillList = [];
    for (const i of list) {
      const nobj = cloneDeep(i);
      const form = get(i, 'form');
      if (form !== '4') {
        fillList.push(nobj);
        continue;
      }
      const match_id = get(i, 'id');
      const extData = await this.matchExtService.fetch({ match_id });
      if (!extData) {
        fillList.push(nobj);
        continue;
      }
      nobj.ext_status = get(extData, 'status');
      fillList.push(nobj);
    }
    return { data: fillList, total };
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_match })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    let result = new FVO_match(data);
    const extData = await this.matchExtService.fetch({ match_id: id });
    result = Object.assign(result, { ext_status: get(extData, 'status', ''), ext_info: get(extData, 'info', []), ext_finals: get(extData, 'finals', []) });
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_match })
  async create(@Body() data: object) {
    const extColumns = ['ext_info', 'ext_finals'];
    const matchData = omit(data, extColumns);
    let extData = pick(data, extColumns);
    const dbData = await this.service.create(matchData);
    // 如果类型是4,需要特殊处理下,补充赛事拓展表
    if (get(matchData, 'form') === '4') {
      // 查查有没有拓展数据,有就不创建了
      const match_id = dbData.id;
      const ed = await this.matchExtService.fetch({ match_id });
      if (!ed) {
        extData = { match_id, info: get(extData, 'ext_info'), finals: get(extData, 'ext_finals') };
        await this.matchExtService.create(extData);
      }
    }
    const result = new CVO_match(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_match })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const extColumns = ['ext_info', 'ext_finals'];
    const matchData = omit(data, extColumns);
    const extData = pick(data, extColumns);
    const result = await this.service.update({ id }, matchData);
    await this.service.updateProject({ id }, data);
    // 如果类型是4,需要特殊处理下,补充赛事拓展表,不需要修改状态和赛事id
    if (get(matchData, 'form') === '4') {
      await this.matchExtService.update({ match_id: id }, { info: get(extData, 'ext_info'), finals: get(extData, 'ext_finals') });
    }
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    await this.service.deleteFlow(id);
    return result;
  }

  @Get('/detail/:id')
  @ApiResponse({ type: FVO_match })
  async detail(@Param('id') id: string) {
    let data = await this.service.fetch({ id });
    data = await this.serviceUtil.fillOnwer(data);
    data = await this.serviceUtil.fillCollection(data, 'match');
    return data;
  }
}
