import { Controller, Inject, Get, Param, Post, Body, Del, Query } from '@midwayjs/core';
import { ProjectService } from '../../service/platform/project.service';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { omit, pick } from 'lodash';
import { ServiceError, ErrorCode } from '../../error/service.error';
import { BaseController } from '../../frame/BaseController';
import { QVO_project, FVO_project, CVO_project, UVAO_project } from '../../interface/platform/project.interface';
import { Context } from '@midwayjs/koa';
import { ServiceUtilService } from '../../service/serviceUtil.service';
const namePrefix = '项目';
@ApiTags(['项目'])
@Controller('/project', { tagName: namePrefix })
export class ProjectController implements BaseController {
  @Inject()
  service: ProjectService;
  @Inject()
  ctx: Context;
  @Inject()
  serviceUtil: ServiceUtilService;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_project })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_project })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_project(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_project })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    const result = new CVO_project(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_project })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }

  @Get('/list')
  async list(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const { data, total } = await this.service.query(qobj, others);
    for (let i of data) {
      i = await this.serviceUtil.fillOnwer(i);
    }
    return { data, total };
  }

  @Get('/detail/:id')
  @ApiResponse({ type: FVO_project })
  async detail(@Param('id') id: string) {
    let data = await this.service.fetch({ id });
    data = await this.serviceUtil.fillOnwer(data);
    data = await this.serviceUtil.fillCollection(data, 'project');
    return data;
  }
}
