import { Body, Controller, Del, Get, Inject, Param, Post, Put, Query } from '@midwayjs/core';
import { AdminService } from '../../service/system/admin.service';
import { get, omit, pick } from 'lodash';
import { ApiResponse, ApiTags } from '@midwayjs/swagger';
import { BaseController } from '../../frame/BaseController';
import { ErrorCode, ServiceError } from '../../error/service.error';
import { Validate } from '@midwayjs/validate';
import { CVO_admin, FVO_admin, QVO_admin, UVAO_admin } from '../../interface/system/admin.interface';
import * as bcrypt from 'bcryptjs';

const namePrefix = '管理用户';
@ApiTags(['管理用户表'])
@Controller('/admin', { tagName: namePrefix })
export class HomeController implements BaseController {
  controllerCode = 'user_admin';
  @Inject()
  service: AdminService;
  @Get('/')
  @ApiTags('列表查询')
  @ApiResponse({ type: QVO_admin })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_admin })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_admin(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_admin })
  async create(@Body() data: object) {
    // 处理密码
    const passowrd = get(data, 'password');
    if (passowrd) {
      const salt = bcrypt.genSaltSync(10);
      const hash = bcrypt.hashSync(passowrd, salt);
      Object.assign(data, { password: hash });
    }
    const result = await this.service.create(data);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_admin })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    data = omit(data, ['password']);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    const result = await this.service.delete({ id });
    return result;
  }

  @Put('/init')
  async init() {
    await this.service.initSuperAdmin();
    return 'ok';
  }
}
