import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/core';
import { BaseController } from '../../frame/BaseController';
import { ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { get, omit, pick } from 'lodash';
import { ErrorCode, ServiceError } from '../../error/service.error';
import { UserService } from '../../service/system/user.service';
import { CVO_user, FVO_user, QVO_user, UVAO_user } from '../../interface/system/user.interface';
import { RoleService } from '../../service/system/role.service';
import * as bcrypt from 'bcryptjs';

const namePrefix = '平台用户';
@ApiTags(['平台用户'])
@Controller('/user', { tagName: namePrefix })
export class UserController implements BaseController {
  controllerCode = 'user_user';
  @Inject()
  service: UserService;

  @Inject()
  roleService: RoleService;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_user })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/list')
  async list(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const { data, total } = await this.service.query(qobj, others);
    for (let i of data) {
      i = await this.roleService.Isaudit(i);
    }
    return { data, total };
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_user })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_user(data);
    return result;
  }

  @Get('/detail/:id')
  @ApiResponse({ type: FVO_user })
  async detail(@Param('id') id: string) {
    const data = await this.service.detail(id);
    const result = new FVO_user(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_user })
  async create(@Body() data: object) {
    await this.service.createExamine(data);
    await this.service.checkPhone(data);
    await this.service.checkEmail(data);
    // 处理密码
    const passowrd = get(data, 'password');
    if (passowrd) {
      const salt = bcrypt.genSaltSync(10);
      const hash = bcrypt.hashSync(passowrd, salt);
      Object.assign(data, { password: hash });
    }
    const dbData = await this.service.create(data);
    const result = new CVO_user(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_user })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    await this.service.checkPhone(data);
    await this.service.checkEmail(data);
    data = omit(data, ['password']);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }
}
