import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/core';
import { ApplyCompanyService } from '../../service/users/applyCompany.service';
import { CVO_applyCompany, FVO_applyCompany, QVO_applyCompany, UVAO_applyCompany } from '../../interface/users/applyCompany.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { BaseController } from '../../frame/BaseController';
import { omit, pick } from 'lodash';
import { ErrorCode, ServiceError } from '../../error/service.error';
import { ServiceUtilService } from '../../service/serviceUtil.service';
const namePrefix = '企业申请';
@ApiTags(['企业申请'])
@Controller('/applyCompany', { tagName: namePrefix })
export class ApplyCompanyController implements BaseController {
  @Inject()
  service: ApplyCompanyService;

  @Inject()
  serviceUtil: ServiceUtilService;

  @Post('/examine', { routerName: `审核${namePrefix}` })
  @ApiTags('信息审核')
  @ApiQuery({ name: 'examine' })
  async examine(@Body('id') id: number, @Body('status') status: string) {
    if (!id || !status) throw new ServiceError(ErrorCode.BODY_ERROR);
    if (status === '1') await this.service.apply(id);
    const result = await this.service.update({ id }, { status });
    return result;
  }

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_applyCompany })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/list')
  async list(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const { data, total } = await this.service.query(qobj, others);
    const list = [];
    for (const i of data) {
      list.push(await this.service.fillName(i));
    }
    return { data: list, total };
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_applyCompany })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_applyCompany(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_applyCompany })
  async create(@Body() data: object) {
    await this.service.createExamine(data);
    const dbData = await this.service.create(data);
    const result = new CVO_applyCompany(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_applyCompany })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }
}
