import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/core';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { BaseController } from '../../frame/BaseController';
import { omit, pick } from 'lodash';
import { ErrorCode, ServiceError } from '../../error/service.error';
import { CompetitionService } from '../../service/users/competition.service';
import { CVO_competition, FVO_competition, QVO_competition, UVAO_competition } from '../../interface/users/competition.interface';
import { ServiceUtilService } from '../../service/serviceUtil.service';
const namePrefix = '创新大赛管理人';
@ApiTags(['创新大赛管理人'])
@Controller('/competition', { tagName: namePrefix })
export class CompetitionController implements BaseController {
  @Inject()
  service: CompetitionService;

  @Inject()
  serviceUtil: ServiceUtilService;
  
  @Post('/examine', { routerName: `审核${namePrefix}` })
  @ApiTags('信息审核')
  @ApiQuery({ name: 'examine' })
  async examine(@Body('id') id: number, @Body('status') status: string) {
    if (!id || !status) throw new ServiceError(ErrorCode.BODY_ERROR);
    const result = await this.service.update({ id }, { status });
    return result;
  }

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_competition })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_competition })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_competition(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_competition })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    this.serviceUtil.fillIdentity(data, 'Competition');
    const result = new CVO_competition(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_competition })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    this.serviceUtil.fillIdentity(data, 'Competition');
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }
}
