import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/core';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { UtilService } from '../../service/util.service';
import { BaseController } from '../../frame/BaseController';
import { omit, pick } from 'lodash';
import { ErrorCode, ServiceError } from '../../error/service.error';
import { ExpertService } from '../../service/users/expert.service';
import { Context } from '@midwayjs/koa';
import { QVO_expert, FVO_expert, CVO_expert, UVAO_expert } from '../../interface/users/expert.interface';
import { ServiceUtilService } from '../../service/serviceUtil.service';
const namePrefix = '专家';
@ApiTags(['专家'])
@Controller('/expert', { tagName: namePrefix })
export class ExpertController implements BaseController {
  @Inject()
  service: ExpertService;
  @Inject()
  ctx: Context;
  @Inject()
  serviceUtil: ServiceUtilService;

  @Inject()
  utilService: UtilService;

  @Post('/examine', { routerName: `审核${namePrefix}` })
  @ApiTags('信息审核')
  @ApiQuery({ name: 'examine' })
  async examine(@Body('id') id: number, @Body('status') status: string) {
    if (!id || !status) throw new ServiceError(ErrorCode.BODY_ERROR);
    const result = await this.service.update({ id }, { status });
    return result;
  }

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_expert })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_expert })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_expert(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_expert })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    this.serviceUtil.fillIdentity(data, 'Expert');
    const result = new CVO_expert(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_expert })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    this.serviceUtil.fillIdentity(data, 'Expert');
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }

  @Get('/list')
  async list(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const { data, total } = await this.service.query(qobj, others);
    for (let i of data) {
      i = await this.serviceUtil.fillOnwer(i);
    }
    return { data, total };
  }

  @Get('/detail/:id')
  @ApiResponse({ type: FVO_expert })
  async detail(@Param('id') id: string) {
    let data = await this.service.fetch({ id });
    data = await this.serviceUtil.fillOnwer(data);
    data = await this.serviceUtil.fillCollection(data, 'expert');
    return data;
  }
}
